#include "fintek_api.h"

void usage(int argc, char *argv[])
{
	fprintf(stderr, "\n\n\n\nFintek SPI Flash Demo API:\n");
	fprintf(stderr, "%s <op> <addr> <data>\n\n", argv[0]);
	fprintf(stderr, "Param List:\n");
	fprintf(stderr, "\top: 0 read / 1 write\n");
	fprintf(stderr, "\taddr: \n");
	fprintf(stderr, "\tdata: String\n");
}

int spi_flash_read_status(unsigned char *flash_status)
{
	int i, status;

	status = SpiSetCs(0, 1);
	if (status)
		return status;

	status = SpiSetCs(0, 0);
	if (status)
		goto err;

	status = SpiWriteData(0, 0x05);
	if (status)
		goto err;

	status = SpiReadData(0, flash_status);
	if (status)
		goto err;

	status = SpiSetCs(0, 1);
	if (status)
		goto err;

	return 0;

 err:
	SpiSetCs(0, 1);
	return -1;
}

int spi_flash_set_write_enable(unsigned int en)
{
	int status;

	status = SpiSetCs(0, 1);
	if (status)
		return status;

	status = SpiSetCs(0, 0);
	if (status)
		goto err;

	status = SpiWriteData(0, en ? 0x06 : 0x04);
	if (status)
		goto err;

	status = SpiSetCs(0, 1);
	if (status)
		goto err;

	return 0;

 err:
	SpiSetCs(0, 1);
	return -1;
}

int spi_flash_sector_erase(unsigned int addr)
{
	int status;

	status = SpiSetCs(0, 1);
	if (status)
		return status;

	/* sector erase */

	status = SpiSetCs(0, 0);
	if (status)
		goto err;

	status = SpiWriteData(0, 0x20);	// erase
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 16) & 0xff);	// addr1
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 8) & 0xff);	// addr2
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 0) & 0xff);	// addr3
	if (status)
		goto err;

	status = SpiSetCs(0, 1);
	if (status)
		goto err;

	return 0;

 err:
	SpiSetCs(0, 1);
	return -1;
}

int spi_flash_read_data(unsigned int addr, unsigned char *data, int size)
{
	int i, status;

	if (size > 256)
		return 0;

	status = SpiSetCs(0, 1);
	if (status)
		return status;

	status = SpiSetCs(0, 0);
	if (status)
		goto err;

	status = SpiWriteData(0, 0x03);	// read
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 16) & 0xff);	// addr1
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 8) & 0xff);	// addr2
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 0) & 0xff);	// addr3
	if (status)
		goto err;

	for (i = 0; i < size; ++i) {
		status = SpiReadData(0, &data[i]);
		if (status)
			goto err;
	}

	status = SpiSetCs(0, 1);
	if (status)
		goto err;

	return 0;

 err:
	SpiSetCs(0, 1);
	return -1;
}

int spi_flash_write_data(unsigned int addr, unsigned char *data, int size)
{
	int i, status;

	if (size > 256)
		return 0;

	status = SpiSetCs(0, 1);
	if (status)
		return status;

	status = SpiSetCs(0, 0);
	if (status)
		goto err;

	status = SpiWriteData(0, 0x02);	// write
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 16) & 0xff);	// addr1
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 8) & 0xff);	// addr2
	if (status)
		goto err;

	status = SpiWriteData(0, (addr >> 0) & 0xff);	// addr3
	if (status)
		goto err;

	for (i = 0; i < size; ++i) {
		status = SpiWriteData(0, data[i]);
		if (status)
			goto err;
	}

	status = SpiSetCs(0, 1);
	if (status)
		goto err;

	return 0;

 err:
	SpiSetCs(0, 1);
	return -1;
}

int spi_flash_is_idle(int retry)
{
	int status;
	unsigned char flash_status;

	while (retry--) {
		status = spi_flash_read_status(&flash_status);
		if (status)
			return status;

		if (!(flash_status & 0x01))
			return 0;
	}

	return -1;
}

int spi_flash_demo(int argc, char *argv[])
{
	unsigned int i, j, op, addr;
	int status;
	unsigned char buf[256];
	int row = 16;

	if (argc < 4) {
		usage(argc, argv);
		exit(3);
	}

	op = strtol(argv[1], NULL, 16);
	addr = strtol(argv[2], NULL, 16);

	status = SpiSetPolPha(0, 0, 0);
	if (status) {
		printf("spi_flash_read_data failed\n");
		return status;
	}

	if (!op) {
		status = spi_flash_read_data(addr, buf, sizeof(buf));
		if (status) {
			printf("spi_flash_read_data failed\n");
			return status;
		}

		for (i = 0; i < sizeof(buf) / row; ++i) {
			for (j = 0; j < row; ++j)
				printf("%02X ", buf[i * row + j]);

			printf("\n");
		}

	} else {
		/* Write */
		status = spi_flash_set_write_enable(1);
		if (status) {
			printf("spi_flash_set_write_enable failed\n");
			return status;
		}

		status = spi_flash_sector_erase(addr);
		if (status) {
			printf("spi_flash_sector_erase failed\n");
			return status;
		}

		status = spi_flash_is_idle(100);
		if (status) {
			printf("spi_flash_is_idle failed\n");
			return status;
		}

		status = spi_flash_set_write_enable(1);
		if (status) {
			printf("spi_flash_set_write_enable failed\n");
			return status;
		}

		printf("Write %s, len: %zu\n", argv[3], strlen(argv[3]));
		status = spi_flash_write_data(addr, argv[3], strlen(argv[3]));
		if (status) {
			printf("spi_flash_read_data failed\n");
			return status;
		}

		status = spi_flash_is_idle(100);
		if (status) {
			printf("spi_flash_is_idle failed\n");
			return status;
		}

		printf("Write complete\n");
	}

	return 0;
}

int main(int argc, char *argv[])
{
	int nRet = 0;
	sFintek_sio_data sio_data;

	/* Please set this to 1 enable more debug message */
	set_debug(0);

	fprintf(stderr, "FINTEK Lib Version: %s\n", getFintekLibVersion());

	if (nRet = init_fintek_sio(eSIO_TYPE_F75115, 0, &sio_data)) {
		fprintf(stderr, "init_fintek_sio error\n");
		exit(3);
	}

	ActiveSIO(sio_data.ic_port, sio_data.key);
	spi_flash_demo(argc, argv);
	DeactiveSIO(sio_data.ic_port);

	return 0;
}
